function [u_sel,a,A]=FRA_PSO_RBF(u,y,n,pst)
%version: 1.0
%date: 05/04/2010
%programmer: Jing Deng
% u: full regression matrix, y: output; n: number of terms to be selected
% u_sel: the optimized RBF centres; a: model coefficients; A: intermediate regression context; 
% pst = [s,l,wi]
%% initialization
%     load test_code.mat
%     load ex1_data.mat
%     u = P(:,1:200)';
%     y = Y(1:200)';

    %%%%%%%%%%%% parameters need to be set %%%%%%%%%%%%
%     s = 15;   %the number of swarms
%     l = 20;   %the maximun number of iterations
%     n = 12;   %the number of selected terms
    s = pst(1);
    l = pst(2);
    wi = pst(3);
    M = n+s;  %used to determin the size of intermediate matrix
%     c1 = 0.8;
%     c2 = 0.8;
%     wi = 0.8;    %the updating parameter in velocity
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    [N,m] = size(u);    %extract the number of data samples and input size
    A=zeros(n,M);   %The intermediate regression matrix same as in the paper
    Ay = zeros(M,1);    %The intermediate regression matrix same as in the paper
    P = [];     %used to store the selected terms;  
    u_sel = [];
    
    %%%%%%%%%  define the search space  %%%%%%%%%%%%%%%
    for i = 1:m     %search space of solution
        u_min(i) = min(u(:,i));   
        u_max(i) = max(u(:,i));
    end
    u_min = [u_min - 0.2.*abs(u_min), 0.1*ones(1,m)];     % add the constrain for the minimum RBF width
    u_max = [u_max + 0.2 .* abs(u_max), 8*ones(1,m)];   % add the constrain for the maximum RBF width
    v_max = 0.5 * (u_max - u_min); %the search space of velocity
    var_rbf = 4*ones(m,s);    %the initial width of RBF
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
   
    v = -v_max'*ones(1,s) + v_max' * 2. * rand(1,s);  %the initial value of velocities ?randomly generated inside the range?
    reduce_final = 0; %used to store the total reduction to the cost function
     
%% start the main selection
    for k = 1:n
        u_s = [];
        rand_ind = floor(N*rand(1,s))+1;  %randomly generate the index of s data samples for initial points
        u_s = [u(rand_ind,:)';var_rbf];    %2m*s used to form the parameters to be optimized centre+width
        p_b = u_s;    %the best position for each swarm

%         u_dist = zeros(N,s);    %the distance of each swarm with other data points
%         for i_phi = 1:n
%             u_dist = u_dist + dist(u_s(i_phi,:)',u(:,i_phi)')'.^2./(ones(N,1)*u_s(n+i_phi,:));
%         end
%         Phi = exp(-0.5*u_dist);
% 
%         Ay=y'*Phi;
%         Norm2=sum(Phi.*Phi);
%         Reduces=Ay.*Ay./Norm2;

        Reduces = net_contribution(P,A,Ay,k,u_s,u,y);
        v_b = Reduces;  %v_b stores the best position of each swarm 
        
        
        [v_g, p_g_i] = max(Reduces);
        p_g = p_b(:,p_g_i);   %the best global position
        
        iter_k = 0;     %the iteration count
        while iter_k < l-1
            iter_k = iter_k + 1;
            for j = 1:s     %check the best position of each swarm
                if v_b(j) < Reduces(j)
                    p_b(:,j) = u_s(:,j);
                    v_b(j) = Reduces(j);
                end                
            end
            [v_g_temp,p_g_i] = max(v_b);     %check the best global position
            if v_g_temp > v_g
                v_g = v_g_temp;
                p_g = p_b(:,p_g_i);
            end
            %update velocity and position
            c1 = (0.5-2.5)*iter_k/l + 2.5;
            c2 = (2.5-0.5)*iter_k/l + 0.5;
            v = wi * v + c1 * rand(2*m,s) .* (p_b - u_s) + c2 * rand(2*m,s) .* (p_g*ones(1,s) - u_s);
            u_s = u_s + v;
            
            [u_s,v] = value_check(u_s,[u_min',u_max'],v,[-v_max',v_max']);
            
           Reduces = net_contribution(P,A,Ay,k,u_s,u,y);
        %v_b = Reduces;  %v_b stores the best position of each swarm 
        
        %[v_g, p_g] = max(Reduces);
        end
        
        u_dist = zeros(N,1);    %the distance of each swarm with other data points (one column of P)
        for i_phi = 1:m
            u_dist = u_dist + dist(p_g(i_phi)',u(:,i_phi)')'.^2./p_g(m+i_phi); %(ones(N,1)*p_g(m+i_phi));
        end
        u_sel = [u_sel,p_g];
        reduce_final = reduce_final + v_g;
        Phi = exp(-0.5*sqrt(u_dist));
        P = [P,Phi];
        for i = 1:k
            A(i,k) = P(:,i)'*Phi;
            for j = 1:i-1
                A(i,k) = A(i,k) - A(j,i)/A(j,j)*A(j,k);
            end
        end
        Ay(k) = Phi'*y;
        for j = 1:k-1
            Ay(k) = Ay(k) - Ay(j)/A(j,j)*A(j,k);
        end   
            
    end 
    A = A(:,1:n);
    Ay = Ay(1:n);
    a = zeros(n,1);
    for k=n:-1:1
		a(k)=Ay(k)-A(k,k+1:n)*a(k+1:n);				% [n-k,n-k]
		a(k)=a(k)/A(k,k);							% [0,1]
	end
%    y_predict = P*a; 
%    mse(y_predict-y)
return

%% The function is to ensure the position and velocity are in the specified range
function [u_o,v_o] = value_check(u_s,u_space,v,v_space)
%u_s n*s
%u_space 2*n [min max]
[n,s] = size(u_s);
for i = 1:s
    for j = 1:n
        if u_s(j,i)>u_space(j,2)
            u_s(j,i) = u_space(j,2);
        end
        if u_s(j,i) < u_space(j,1)
            u_s(j,i) = u_space(j,1);
        end
        if v(j,i)>v_space(j,2)
            v(j,i) = v_space(j,2);
        end
        if v(j,i) < v_space(j,1)
            v(j,i) = v_space(j,1);
        end
        if 0 < v(j,i) < 0.3
            v(j,i) = rand(1) * 0.5 * v_space(j,2);
        end
        if  -0.3 < v(j,i) < 0
            v(j,i) = rand(1) * 0.5 * v_space(j,1);
        end
    end
end
u_o = u_s;
v_o = v;
return

% %% this function is used to calculate the contribution of each swarm
% function Reduces = contribution(P,A,Ay,k,u_s,u,y)
% [N,n] = size(u);
% s = length(u_s(1,:));
% u_dist = zeros(N,s);    %the distance of each swarm with other data points
% for i_phi = 1:n
%      u_dist = u_dist + dist(u_s(i_phi,:)',u(:,i_phi)')'.^2./(ones(N,1)*u_s(n+i_phi,:));
% end
% Phi = exp(-0.5*sqrt(u_dist));
% 
% if k == 1
%     Ay=y'*Phi;
%     Norm2=sum(Phi.*Phi);
%     Reduces=Ay.*Ay./Norm2;
% else
%     % calculate the k-th column of matrix A
%     for i = 1:k-1
%         A(i,k:k+s-1) = P(:,i)'*Phi;
%         for j = 1:i-1
%             A(i,k:k+s-1) = A(i,k:k+s-1) - A(j,i)/A(j,j)*A(j,k:k+s-1);
%         end
%     end
%     A(k,k:k+s-1) = sum(Phi.*Phi);
%     for j = 1:k-1
%         A(k,k:k+s-1) = A(k,k:k+s-1) - A(j,k:k+s-1)/A(j,j).*A(j,k:k+s-1);
%     end
%     
%     %calculate the k-th elements of matrix Ay
%     Ay(k:k+s-1) = Phi'*y;
%     for j = 1:k-1
%         Ay(k:k+s-1) = Ay(k:k+s-1) - Ay(j)/A(j,j)*A(j,k:k+s-1)';
%     end
%     Reduces=Ay(k:k+s-1)'.^2./A(k,k:k+s-1);
% end
%         
% return;
% 
