function [p,y,p_reg] = RegressionMatrix(y,u,order,max_power)
% first Programmed by Jing on 06-May-2009, 
%revised on 01/March/2010
%revised on 03/March/2010
% [p,y] = RegressionMatrix(data,p_reg)
%function is used to form the regression matrix for term selection
%altorithm. It is approprite for MISO system. 
%y and u are the original output and inputs data
% order = [na,nb,nk]; max_power is the maximum power of polynomial terms

%% the part of code is used to get the information from the data
% Num_inputs = size(data.InputData,2);  
Num_outputs = size(y,2);
if Num_outputs > 1
    error('This function only supports MISO model')
end

data=iddata(y,u,1);

nu = (length(order)-1)/2; % get how many input variables
na = order(1);
nb = order(2:1+nu);

if nu>15
    error('Maximum input variables is 15')
end

nk = order(2+nu:end);
max_delay = max(nb+nk);  %decide the first sample to form the regression matrix

N = size(y,1);  % the size of data sample

%start to form the nonlinear polynomial terms
M = idnlarx(order,'linear');
if max_power > 1
    p1 = polyreg(M, 'MaxPower',max_power,'CrossTerm','on');
    M2 = addreg(M,p1);
else
    M2 = M;
end

p_reg = getreg(M2);
m = length(p_reg);
if N < m+max_delay
    warning('The data is too less');
end

phi = getreg(M2,'all',data);
p = phi(max_delay:end,:);
y = y(max_delay:end);
