function [u_sel,a,A]=ident_fra_pso_loo_rbf(u,y,pst)
%version: 0.1
%date: 11/01/2011
%programmer: Jing Deng
% u: input matrix,  N*m   m is the input dimension
% y: output ;
% u_sel: the optimized RBF centres; a: model coefficients; A: intermediate regression context; 
% pst = [s,l,wi]
%% initialization
%     load bm1_0.05_noise.mat;
%     u = P(1:200);
%     y = Y(1:200);
%     s = 15;   %the number of swarms
%     l = 20;   %the maximun number of iterations
%     wi = 0.8;    %the updating parameter in velocity
    %%%%%%%%%%%% parameters need to be set %%%%%%%%%%%%

%    n = 12;   %the number of selected terms
%     wi = 0.8; 
    N = length(y);
    s = pst(1);
    l = pst(2);
    wi = pst(3);
    M = N+s;  % possible columns of intermediate matrix
    c1 = 0.8;
    c2 = 0.8;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    [N,m] = size(u);    %extract the number of data samples and input size
    A=zeros(M,M);   %The intermediate regression matrix same as in the paper
    b = zeros(M,1);    %The intermediate regression matrix same as in the paper
    P = [];     %used to store the selected terms;  
    P_im = []; % to store \mathbf{p}_i^{(i-1)} for selected terms
    u_sel = []; %The final selected RBF centres
    
    %%%%%%%%%  define the search space  %%%%%%%%%%%%%%%
%     for i = 1:m     %search space of solution
%         u_min(i) = min(u(:,i));   
%         u_max(i) = max(u(:,i));
%     end

    % define the search space of solution, including centre and width [position, width]
    u_min = min(u,[],1);
    u_max = max(u,[],1);
    u_min = [u_min - 0.2.*abs(u_min), 0.1*ones(1,m)];     % add the constrain for the minimum RBF width
    u_max = [u_max + 0.2 .* abs(u_max), 8*ones(1,m)];   % add the constrain for the maximum RBF width
    
    % define the search space of velocity for both position and RBF width
    v_max = 0.5 * (u_max - u_min); % based on the solution space
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    var_rbf = 2*ones(m,s);    % the initial width of RBF centre
   
    reduce_final = 0; % used to store the total reduction to the cost function
    J_0 = mean(y.^2)+0.00001;  %make sure the initial value of J_k0 is large than J_k1, so the following cycle can start
    J_1 = mean(y.^2);
    stop_flag = 1; % terminate selection process
    k = 0; % number of selected terms
    r_im = ones(N,1);
    e_im = y;
%% The first stage of forward construction
    while stop_flag
        k = k+1;
        J_0 = J_1;
        clear u_s; % empty swarms for a new selection step
        rand_ind = ceil(N*rand(1,s));  % randomly generate the index of s data samples for initial points
        u_s = [u(rand_ind,:)';var_rbf];    % 2m*s used to form the parameters to be optimized centre+width, s swarms and each swarm has 2m parameters
        p_b = u_s;    % the best position for each swarm, obviously the best are themselves at first
        v = - v_max' * ones(1,s) + v_max' * 2. * rand(1,s);  % the initial value of velocities ?randomly generated inside the range?
        loo_error = loo_error_calculation(P,P_im,r_im,e_im,A,b,k,u_s,u,y); % caculate the net contribution of each particle to the cost function
        v_b = loo_error;  % v_b stores the best value of loo error for each particle, they are themselves at first step
        
        
        [v_g, p_g_i] = min(loo_error); % find the global best value
        p_g = p_b(:,p_g_i);   % the best global position
        
        iter_k = 0;     %the iteration count
        
        while iter_k < l-1 % l is the maximum iteration of updating in the swarm
            iter_k = iter_k + 1; % count the iteration
            
            % check the best position of each particle
            for j = 1:s     
                if v_b(j) > loo_error(j)
                    p_b(:,j) = u_s(:,j); % store position
                    v_b(j) = loo_error(j); % store best loo error or each particle
                end                
            end
            
            %check the best global position
            [v_g_temp,p_g_i] = min(v_b);     
            
            if v_g_temp < v_g
                v_g = v_g_temp; % store best net contribution
                p_g = p_b(:,p_g_i); % store best solution
            end
            
            %update velocity and position
            c1 = (0.5-2.5)*iter_k/l + 2.5;
            c2 = (2.5-0.5)*iter_k/l + 0.5;
            v = wi * v + c1 * rand(2*m,s) .* (p_b - u_s) + c2 * rand(2*m,s) .* (p_g*ones(1,s) - u_s);
            u_s = u_s + v;
            
            [u_s,v] = value_check(u_s,[u_min',u_max'],v,[-v_max',v_max']); % check range
            
            % calculate the new net contributions of new particles
           loo_error = loo_error_calculation(P,P_im,r_im,e_im,A,b,k,u_s,u,y);
        
        end
        
%         u_dist = zeros(N,1);    %the distance of each swarm with other data points (one column of P)
%         for i_phi = 1:m
%             u_dist = u_dist + dist(p_g(i_phi)',u(:,i_phi)')'.^2./p_g(m+i_phi); %(ones(N,1)*p_g(m+i_phi));
%         end
        u_mu = p_g(1:m)'; % the mean matrix used by Gaussian mixture distribution
        u_sigma(1,:,:) = p_g(m+1:end); % the variance matrix used by Gaussian mixture distribution
        obj = gmdistribution(u_mu,u_sigma); % used to compute Mahalanobis distance
        d_mahal = mahal(obj,u); % compute the Mahalanobis distance
        phi = exp(-0.5 * sqrt(d_mahal)); % produce the regression matrix

        u_sel = [u_sel,p_g];
        reduce_final = [reduce_final, v_g];
        
        J_1 = v_g;
%         Phi = exp(-0.5*sqrt(u_dist));
        P = [P,phi];
        % update A and b
        phi_im = phi;
        for i = 1:k-1
            A(i,k) = P_im(:,i)'*phi;
            phi_im = phi_im - P_im(:,i) * A(i,k) ./ A(i,i);
        end
        A(k,k) = phi_im'*phi;
        
%         if k > 1
%             for i = 2:k-1
%                 A(i,k) = P_im(:,i)'*phi;
%                 phi_im = phi_im - P_im(:,i) * A(i-1,k) ./ A(i-1,i-1);
%             end
%             phi_im = phi_im - P_im(:,k-1) * A(k-1,k)./A(k-1,k-1);
%             A(k,k) = phi_im'*phi;
%         end
%         
        P_im = [P_im,phi_im];
        b(k) = phi_im'*y;
        r_im = r_im - P_im(:,k).^2./A(k,k);
        e_im = e_im - (b(k)/A(k,k))*P_im(:,k);
%         if k == 1
%             R_im(:,k) = R_im(:,k) - P_im.^2./A(k,k);
%             E_im(:,k) = R_im(:,k) - (b(k)/A(k,k)).*P_im;
%         else
%             R_im(:,k) = R_im(:,k-1) - P_im.^2./A(k,k);
%             E_im(:,k) = R_im(:,k-1) - (b(k)/A(k,k)).*P_im;
%         end
%         for i = 1:k
%             A(i,k) = P(:,i)'*Phi;
%             for j = 1:i-1
%                 A(i,k) = A(i,k) - A(j,i)/A(j,j)*A(j,k);
%             end
%         end
%         
%         Ay(k) = Phi'*y;
%         for j = 1:k-1
%             Ay(k) = Ay(k) - Ay(j)/A(j,j)*A(j,k);
%         end  
  %%% determin the stop point  %%%%%
    if J_0 > J_1 && (J_0 - J_1)/mean(y.^2) < 2e-3
        stop_flag = 0;
    end
    if J_0 < J_1 && (J_1 - J_0)/mean(y.^2) > 2e-3
        stop_flag = 0;
    end
            
end
    n = k;
    A = A(1:n,1:n);
    b = b(1:n);
    a = zeros(n,1);
    for k=n:-1:1
		a(k)=b(k)-A(k,k+1:n)*a(k+1:n);				% [n-k,n-k]
		a(k)=a(k)/A(k,k);							% [0,1]
	end
%    y_predict = P*a; 
%    mse(y_predict-y)
return


%% The function is to ensure the position and velocity are in the specified
%% range
function [u_o,v_o] = value_check(u_s,u_space,v,v_space)
%u_s n*s
%u_space 2*n [min max]
[n,s] = size(u_s);
for i = 1:s
    for j = 1:n
        if u_s(j,i)>u_space(j,2)
            u_s(j,i) = u_space(j,2);
        end
        if u_s(j,i) < u_space(j,1)
            u_s(j,i) = u_space(j,1);
        end
        if v(j,i)>v_space(j,2)
            v(j,i) = v_space(j,2);
        end
        if v(j,i) < v_space(j,1)
            v(j,i) = v_space(j,1);
        end
        if 0 < v(j,i) < 0.3
            v(j,i) = rand(1) * 0.5 * v_space(j,2);
        end
        if  -0.3 < v(j,i) < 0
            v(j,i) = rand(1) * 0.5 * v_space(j,1);
        end
    end
end
u_o = u_s;
v_o = v;
return
