function [index,theta,aicy] = ident_ols(p,y)
% OLS algorithm by using classic Gram-Schmidt method
%output: index contains the terms that are selected by the algorithm
%output: theta is the associated parameters
%output: aicy records the decrease of LOO error
%Input: P is the regression matrix
% y is the output data
% note: this code is for multi-input and single output model only.

[n,m] = size(p);  % n is the number of data samples, m is the terms of possible combination

% z = y(4:n+3); % The maximal delay is 3,so the system output starts from the 4th sample 
z = y;
phi = p;

a = eye(m); % a is an unit upper triangular matrix
p_aic = 2;

comp = 0; % A temporary variable for chosing the a term which gives the maximal contribution to the finial model.

s = 1; 
aic_0 = n*log((y'*y)/n);
aic_y = n*log((y'*y)/n);
aic_init = aic_0;
%%%%%%%%%% the first stage of OLS algorithm %%%%%%%%%%%%%%%%
% select the one column from regressive matrix P for orthogonal matrix W
for i=1:m
    w_1(:,i) = phi(:,i);
    g_1(i) = (w_1(:,i)'*z)/(w_1(:,i)'*w_1(:,i)); 
    err_1(i) = (g_1(i)^2*(w_1(:,i)'*w_1(:,i)))/(z'*z);
    
    % compare the error of each term, and select the maximal one. At the
    % same time, assign the value of selected term to final W, g and error
    % kb is used to store the column number of selected term
    if err_1(i)>comp   
        w(:,s) = w_1(:,i);
        g(s) = g_1(i);
        err(s) = err_1(i);
        kb(s) = i;
        comp = err_1(i);
    end
    
end

err_sum = err(1); %  and the error of each selection to the total error

sse = (1-err_sum)*(y'*y);
aic_y = n*log(sse/n)+p_aic*s;
aicy(s) = aic_y;

%%%%%%%%%%%% the second stage of OLS algorithm %%%%%%%%%%%%%%%%%%%
%  if the sum of error is big engough, the algorithm will stop, here the
%  stop criterion is 1-0.00012;

% while 1 - err_sum > creteria
% while aic_y  > creteria 
%  s<creteria
stop_flag = 1;
while stop_flag
    s = s+1; 
    aic_0 = aic_y;
    comp = 0;  % reset the compare variable
    err_1 = [ ]; % reset the variable which was used to store the temporary error
    
    for i = 1:m
        flg = 0; % flag is a temporary variable

        % start to check if the term of i has been selected. If it was
        % selected, then go to next loop. 
        for j = 1:s-1
            if i == kb(j)
                flg = 1;
                 break;
            end
        end
        
        if flg == 1
            continue;
        end
       % end of checking
       
        w_a = 0; % temporary variable
        for j = 1:s-1
            a_1(j) = (w(:,j)'*phi(:,i))/(w(:,j)'*w(:,j));
            w_a = a_1(j)*w(:,j)+w_a;
        end
        w_1(:,i) = phi(:,i) - w_a;
        g_1(i) = (w_1(:,i)'*z)/(w_1(:,i)'*w_1(:,i));
        err_1(i) = (g_1(i)^2*(w_1(:,i)'*w_1(:,i)))/(z'*z);
        
        if err_1(i)>comp  % find out the term which gives largest contribution to the final model
            w(:,s) = w_1(:,i);
            g(s) = g_1(i);
            err(s) = err_1(i);
            for j = 1:s-1
                a(j,s) = a_1(j);
            end
            kb(s) = i;
            comp = err_1(i);
        end
        
    end
     err_sum = err_sum + err(s);
     sse = (1-err_sum)*(y'*y);
     aic_y = n*log(sse/n)+p_aic*s;
     aicy(s) = aic_y;
     
     if s > m-1  % if all terms are checked, then stop algorithm
         break;
     end

    % determin the stop point  %%%%%
     if aic_0 > aic_y && abs((aic_0 - aic_y)/aic_init)  < 1e-3
        stop_flag = 0;
    end
    if aic_0 < aic_y && abs((aic_y - aic_0)/aic_init)  > 1e-3
        stop_flag = 0;
    end
end

a_f = a(1:s-1, 1:s-1); % the final a only consist the first s rows and columns
index = kb(1:s-1); % used to display the number of columns which were selected
theta = inv(a_f)*g(1:s-1)'; % used to display the estimated parameters corresponding to each selected term

% the following code was used to verify the model created by above algorithm

% y_v(1:8) = y1(n+1:n+8);
% 
% for i = 9:2000
% y_v(i) = [y_v(i-1),u2(n+i-8),u2(n+i-13),u2(n+i-10),u2(n+i-11),u2(n+i-12),u2(n+i-24),u2(n+i-19),u1(n+i-11),u1(n+i-10)]*theta(1:10);
% end
% t = 1:2000;
% plot(t,y_v,t,y1(n+1:n+2000))
% title('Speed.(sim)');
% xlabel('data samples');
% ylabel('speed (r/min)');
% 
% title('Intake Manifold Pressure.(sim)');
% xlabel('data samples');
% ylabel('Intake Manifold Pressure (bar)');

% 
% title('Intake Air Temperature.(sim)');
% xlabel('data samples');
% ylabel('Intake Air Temperature (oC)');

% y_id = y;
% 
% for i = 4:n+3
%     y_id(i) = theta'*[y(i-1); y(i-2); u(i-1)^3; u(i-2)*y(i-3)];
%     err_id(i) = y_id(i) - y(i);
% end
% z_id = y_id(4:n+3);
% t = 4:n+3;
% err_id = err_id(4:n+3);
% 
% subplot(2,1,1)
% plot(t,z,':b',t,z_id,'-r' );
% xlabel('time / 0.2s')
% ylabel('output')
% title('one-step-ahead prediction (red line) compared with actual output (blue line)')
% subplot(2,1,2)
% plot(t,err_id)
% xlabel('time / 0.2s')
% ylabel('error')
% title('error of predicted output and actual output')
%     
