function [index,theta,a_f,sse_train] = ident_ols_loo(p,y)
% OLS algorithm by using classic Gram-Schmidt method and leave one out
% method


[n,m] = size(p);  % n is the number of data samples, m is the terms of possible combination
a = eye(m); % a should be an unit upper triangular matrix
eps0 = y;  % the initial residual
beta0 = ones(n,1);  %the denominator of LOO error
w = []; % the orthogonal matrix
sel = [];  %the orders that were selected
k = 1;  % the order of selected terms
term_pool = 1:m;  % store the terms that were not selected
gam = [];   %store the model parameters
sse_train = [];
%% the first stage
J1_i = zeros(m,1);
for i = 1:m
    w1_i = p(:,i);
    k1_i = w1_i'*w1_i;
    gam1_i = (w1_i'*y)/(w1_i'*w1_i);
    eps1_i = eps0 - w1_i * gam1_i;
    beta1 = beta0 - w1_i.^2./k1_i;
    j_res = (eps1_i./beta1).^2;
    J1_i(i) = mean(j_res);
end

[val, ind] = min(J1_i);

w = [w,p(:,term_pool(ind))];                                %to be updated 1
J_k1 = val;                                                 %to be updated 2
sel = [sel,term_pool(ind)];                                 %to be updated 3
term_pool(ind) = [];                                        %to be updated 4
gam1 = (w(:,1)'*y/(w(:,1)'*w(:,1)));
gam = [gam,gam1];                                           %to be updated 5
eps_k1 = eps0 - w(:,1)*gam1;                                %to be updated 6
beta_k1 = beta0 - w(:,1).^2/(w(:,1)'*w(:,1));               %to be updated 7
sse_train = [sse_train, sse(eps_k1)];
%% the second stage
k = k+1;
J2_i = zeros(m-k+1,1);
for i = 1:m-k+1
    a_12 = w(:,1)'* p(:,term_pool(i))/(w(:,1)'*w(:,1));
    wk_i = p(:,term_pool(i)) - a_12 * w(:,1);
    kk_i = wk_i'*wk_i;
    gamk_i = (wk_i'*y)/(wk_i'*wk_i);
    epsk_i = eps_k1 - wk_i * gamk_i;
    betak = beta_k1 - wk_i.^2./kk_i;
    J2_i(i) = mean(epsk_i.^2./(betak.^2));
end
[val, ind] = min(J2_i);
sel = [sel,term_pool(ind)];                                 %to be updated 3
term_pool(ind) = [];                                        %to be updated 4

a_12 = w(:,1)'* p(:,sel(k))/(w(:,1)'*w(:,1));
a(1,2) = a_12;
wk_i = p(:,sel(k)) - a_12 * w(:,1);
kk_i = wk_i'*wk_i;
gamk_i = (wk_i'*y)/(wk_i'*wk_i);
epsk_i = eps_k1 - wk_i * gamk_i;
betak = beta_k1 - wk_i.^2./kk_i;

w = [w,wk_i];                                %to be updated 1
J_k = val;                                                  %to be updated 2

gam = [gam,gamk_i];                                           %to be updated 5
eps_k = epsk_i;                               %to be updated 6
beta_k = betak;              %to be updated 7
sse_train = [sse_train, sse(eps_k)];

if J_k > J_k1
    index = sel;
    a_f = a(1,1);
    theta = gam(1);
end
    
%% the third to the end stage  
% 
stop_flag = 1;
while stop_flag
    
    if k == m
        break;
    end
    
    k = k+1;
    J_k1 = J_k;
    eps_k1 = eps_k;
    beta_k1 = beta_k;
    
    Jk_i = zeros(m-k+1,1);
    for i = 1:m-k+1
        aw = 0;   % temporary variable
        for j = 1:k-1
            a_jk = w(:,j)'*p(:,term_pool(i))/(w(:,j)'*w(:,j));
            aw = aw + a_jk*w(:,j);
        end
        wk_i = p(:,term_pool(i)) - aw;
        kk_i = wk_i'*wk_i;
        gamk_i = (wk_i'*y)/(wk_i'*wk_i);
        epsk_i = eps_k1 - wk_i * gamk_i;
        betak = beta_k1 - wk_i.^2./kk_i;
        Jk_i(i) = mean(epsk_i.^2./(betak.^2));
    end
    [val, ind] = min(Jk_i);
    sel = [sel,term_pool(ind)];                                 %to be updated 3
    term_pool(ind) = [];                                        %to be updated 4
    
    aw = 0;   % temporary variable
    for j = 1:k-1
        a_jk = w(:,j)'*p(:,sel(k))/(w(:,j)'*w(:,j));
        a(j,k) = a_jk;
        aw = aw + a_jk*w(:,j);
    end
    wk_i = p(:,sel(k)) - aw;
    kk_i = wk_i'*wk_i;
    gamk_i = (wk_i'*y)/(wk_i'*wk_i);
    epsk_i = eps_k1 - wk_i * gamk_i;
    betak = beta_k1 - wk_i.^2./kk_i;
        
    w = [w,wk_i];                                %to be updated 1
    J_k = val;                                                  %to be updated 2
  
    gam = [gam,gamk_i];                                           %to be updated 5
    eps_k = epsk_i;                               %to be updated 6
    beta_k = betak;              %to be updated 7
    sse_train = [sse_train, sse(eps_k)];
    
        %% determin the stop point  %%%%%
    if J_k1 > J_k && (J_k1 - J_k)/mean(y.^2) < 1e-3
        stop_flag = 0;
    end
    if J_k1 < J_k && (J_k - J_k1)/mean(y.^2) > 1e-3
        stop_flag = 0;
    end
    
%     if J_k1 < J_k 
%         stop_flag = 0;
%     end

end

%%
index = sel;
a_f = a(1:k,1:k);
theta = (a_f)\gam';












