function [u_sel,a,A,loo_record]=ident_tss_de_loo_rbf(u,y,pst)
%version: 0.1
%date: 11/01/2011
%programmer: Jing Deng
% u: input matrix,  N*m   m is the input dimension
% y: output ;
% u_sel: the optimized RBF centres; a: model coefficients; A: intermediate regression context; 
% pst = [s,l,wi]
%% initialization
%     load bm1_0.05_noise.mat;
%     u = P(1:200);
%     y = Y(1:200);
%     s = 15;   % pupulation size
%     l = 20;   % the maximun number of iterations
%     F = 0.8;    %the updating parameter in velocity
%     Cr = 0.6;  % crossover constant
    %%%%%%%%%%%% parameters need to be set %%%%%%%%%%%%    
    s = pst(1);
    l = pst(2);
    F = pst(3);
    Cr = pst(4);
    N = length(y);
    M = N+s;  % possible columns of intermediate matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    [N,m] = size(u);    %extract the number of data samples and input size
    A=zeros(M,M);   %The intermediate regression matrix same as in the paper
    b = zeros(M,1);    %The intermediate regression matrix same as in the paper
    P = [];     %used to store the selected terms;  
    P_im = []; % to store \mathbf{p}_i^{(i-1)} for selected terms
    u_sel = []; %The final selected RBF centres
    %%%%%%%%%  define the search space  %%%%%%%%%%%%%%%
    % define the search space of solution, including centre and width [position, width]
    u_min = min(u,[],1);
    u_max = max(u,[],1);
    u_min = [1.5*u_min,0.1*ones(1,m)];     % add the constrain for the minimum RBF width
    u_max = [1.5*u_max,pst(5)*ones(1,m)];   % add the constrain for the maximum RBF width
    
    % define the search space of velocity for both position and RBF width
    v_max = 0.5 * (u_max - u_min); % based on the solution space
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
    loo_record = []; % used to record loo error at each step
    J_0 = mean(y.^2)+0.00001;  %make sure the initial value of J_k0 is large than J_k1, so the following cycle can start
    J_1 = mean(y.^2);
    stop_flag = 1; % terminate selection process
    k = 0; % number of selected terms
    r_im = ones(N,1);
    e_im = y;
%% The first stage of forward construction
    while stop_flag
        k = k+1;
        J_0 = J_1;
        clear u_s; % empty population vectors for a new selection step
%         rand_ind = ceil(N*rand(1,s));  % randomly generate the index of s data samples for initial points
        rand_perm = randperm(N); % another method to generate random indeces
        rand_ind = rand_perm(1:s);
        var_rbf = 4*rand(m,s);    % the initial width of RBF centre
        u_s = [u(rand_ind,:)';var_rbf];    % 2m*s used to form the parameters to be optimized centre+width, s swarms and each swarm has 2m parameters
        u_mutation = zeros(2*m,s); % used to store the mutated vectors
        u_trial = u_mutation; % used to store the trial vectors
        loo_error = loo_error_calculation(P,P_im,r_im,e_im,A,b,k,u_s,u,y); % caculate the net contribution of each particle to the cost function
        
        iter_k = 0;     %the iteration count
        
        while iter_k < l % l is the maximum iteration of updating in the swarm
            iter_k = iter_k + 1; % count the iteration
            
           % start to generate the new population
            for i = 1:s
                 % Mutation
                rand_perm = randperm(s); % generate the index of three mutually different vectors
                rand_ind = rand_perm(1:3);
                u_mutation(:,i) = u_s(:,rand_ind(1)) + F * (u_s(:,rand_ind(2)) - u_s(:,rand_ind(3)));
                u_mutation(m+1:2*m,:) = abs(u_mutation(m+1:2*m,:)); % to make sure the width vector are posotive
                % Crossover
                br = randi(2*m); % to make sure at least one crossover is executed
                for j = 1:2*m
                    if (rand < Cr) || (j == br)
                        u_trial(j,i) = u_mutation(j,i);
                    else
                        u_trial(j,i) = u_s(j,i);
                    end
                end                
            end
           % Selection
           loo_error_trial = loo_error_calculation(P,P_im,r_im,e_im,A,b,k,u_trial,u,y); % evaluate the loo error for trial vectors
           for i = 1:s
               if loo_error(i) > loo_error_trial(i) % select the one that gives lower loo error
                   u_s(:,i) = u_trial(:,i);
                   loo_error(i) = loo_error_trial(i);
               end
           end
            
            
           % [u_s,v] = value_check(u_s,[u_min',u_max'],v,[-v_max',v_max']); % check range
        end
        
        [loo_min, ind_min] = min(loo_error);
        u_mu = u_s(1:m,ind_min)'; % the mean matrix used by Gaussian mixture distribution
        u_sigma(1,:,:) = u_s(m+1:end,ind_min); % the variance matrix used by Gaussian mixture distribution
        obj = gmdistribution(u_mu,u_sigma); % used to compute Mahalanobis distance
        d_mahal = mahal(obj,u); % compute the Mahalanobis distance
        phi = exp(-0.5 * sqrt(d_mahal)); % produce the regression matrix

        u_sel = [u_sel,u_s(:,ind_min)];
        loo_record = [loo_record, loo_min];
        
        J_1 = loo_min;
%         Phi = exp(-0.5*sqrt(u_dist));
        P = [P,phi];
        % update A and b
        phi_im = phi;
        for i = 1:k-1
            A(i,k) = P_im(:,i)'*phi;
            phi_im = phi_im - P_im(:,i) * A(i,k) ./ A(i,i);
        end
        A(k,k) = phi_im'*phi;
                 
        P_im = [P_im,phi_im];
        b(k) = phi_im'*y;
        r_im = r_im - P_im(:,k).^2./A(k,k);
        e_im = e_im - (b(k)/A(k,k))*P_im(:,k);

  %%% determin the stop point  %%%%%
    if J_0 > J_1 && (J_0 - J_1)/mean(y.^2) < 1e-4
        stop_flag = 0;
    end
    if J_0 < J_1 && (J_1 - J_0)/mean(y.^2) > 1e-4
        stop_flag = 0;
    end
            
end
n = k;%         if k == 1
%             R_im(:,k) = R_im(:,k) - P_im.^2./A(k,k);
%             E_im(:,k) = R_im(:,k) - (b(k)/A(k,k)).*P_im;
%         else
%             R_im(:,k) = R_im(:,k-1) - P_im.^2./A(k,k);
%             E_im(:,k) = R_im(:,k-1) - (b(k)/A(k,k)).*P_im;
%         end
%         for i = 1:k
%             A(i,k) = P(:,i)'*Phi;
%             for j = 1:i-1
%                 A(i,k) = A(i,k) - A(j,i)/A(j,j)*A(j,k);
%             end
%         end
%         
%         Ay(k) = Phi'*y;
%         for j = 1:k-1
%             Ay(k) = Ay(k) - Ay(j)/A(j,j)*A(j,k);
%         end  
%% second stage of network refinement
check_flag = 1; % if this equals zero, then no change happened in last checking loop
count_cycle = 0; % count the number of refinement cycles
loo_last = J_1;
while check_flag==1
    check_flag = 0; % reset the flag
    count_cycle = count_cycle + 1; % count refinement cycles
    for k = n-1:-1:1 % remove the interested term from the one next to last, refer to the original two-stage paper 2006
        for q = k:n-1
            A_q = zeros(1,n); % temporary variable to store the new q-th row of matrix A
            A_q1 = A_q; % temporary variable to store the new q+1-th row of matrix A
                
            A_q(q+1) = A(q+1,q+1) + A(q,q+1)^2/A(q,q);     % new a_{q,q}, the correct index will be revised later           
            A_q(q) = A(q,q+1);     % new a_{q,q+1}, the correct index will be revised later 
            if q < n-1
                A_q(q+2:n) = A(q+1,q+2:n) + A(q, q+1)/A(q,q)*A(q,q+2:n);     % new a_{q,j} j from q+1 to n             
            end
            
            A_q1(q) = A(q,q) - A(q,q+1)^2/A_q(q+1);     % new a_{q+1,q+1}, the correct index will be revised later
            if q < n-1
                A_q1(q+2:n) = A(q, q+2:n) - A(q, q+1)/A_q(q+1) * A_q(q+2:n);     % new a_{q+1,j} j from q+1 to n
            end
            
            b_q = b(q+1) + A(q, q+1) * b(q)/A(q,q);     % new ay_q
            b_q1 = b(q) - A(q, q+1) * b_q /A_q(q+1);     % new ay_{q+1}
                
            p_q = P_im(:,q+1) + A(q, q+1) / A(q, q) * P_im(:,q); %new P_q^{(q-1)}
            p_q1 = P_im(:,q) - A(q, q+1) / A_q(q+1) * p_q; %new P_{q+1}^{(q)}
                              
            A([q,q+1],1:n) = [A_q;A_q1]; % assign new value
            b([q,q+1]) = [b_q,b_q1];
            P_im(:,[q+1,q]) = [p_q1, p_q];
                
%                 iOrder([q,q+1]) = iOrder([q+1,q]); %shift two adjacent terms  
%                 Ad(iOrder(1:n)) = diag(A(1:n,iOrder(1:n)));
            A(:,[q,q+1]) = A(:,[q+1,q]); % restore the correct column index
            P(:,[q+1,q]) =  P(:,[q,q+1]); % interchange the interest term
            u_sel(:,[q+1,q]) = u_sel(:,[q,q+1]); % interchange
        end
        
        clear u_s; % empty population vectors for a new selection step
%         rand_ind = ceil(N*rand(1,s));  % randomly generate the index of s data samples for initial points
        rand_perm = randperm(N); % another method to generate random indeces
        rand_ind = rand_perm(1:s);
        var_rbf = 4*rand(m,s);    % the initial width of RBF centre
        u_s = [u(rand_ind,:)';var_rbf];    % 2m*s used to form the parameters to be optimized centre+width, s swarms and each swarm has 2m parameters
        %u_s(:,1) = u_sel(:,n);
        u_mutation = zeros(2*m,s); % used to store the mutated vectors
        u_trial = u_mutation; % used to store the trial vectors
  
        r_im0 = r_im + P_im(:,n).^2./A(n,n); % new \mathbf{r}_{n-1}
        e_im0 = e_im + (b(n)/A(n,n))*P_im(:,n); % new \mathbf{e}_{n-1}
        
        loo_error = loo_error_calculation(P(:,1:n-1), P_im(:,1:n-1),r_im0,e_im0,A(:,1:n-1),b(1:n-1),n,u_s,u,y);
        
        iter_k = 0;     %the iteration count
        
        while iter_k < l-1 % l is the maximum iteration of updating in the swarm
            iter_k = iter_k + 1; % count the iteration
            
           % start to generate the new population
            for i = 1:s
                 % Mutation
                rand_perm = randperm(s); % generate the index of three mutually different vectors
                rand_ind = rand_perm(1:3);
                u_mutation(:,i) = u_s(:,rand_ind(1)) + F * (u_s(:,rand_ind(2)) - u_s(:,rand_ind(3)));
                u_mutation(m+1:2*m,:) = abs(u_mutation(m+1:2*m,:)); % to make sure the width vector are posotive
                % Crossover
                br = randi(2*m); % to make sure at least one crossover is executed
                for j = 1:2*m
                    if (rand < Cr) || (j == br)
                        u_trial(j,i) = u_mutation(j,i);
                    else
                        u_trial(j,i) = u_s(j,i);
                    end
                end                
            end
           % Selection
           loo_error = loo_error_calculation(P(:,1:n-1), P_im(:,1:n-1),r_im0,e_im0,A(:,1:n-1),b(1:n-1),n,u_s,u,y); % evaluate the loo error for trial vectors
           for i = 1:s
               if loo_error(i) > loo_error_trial(i) % select the one that gives lower loo error
                   u_s(:,i) = u_trial(:,i);
                   loo_error(i) = loo_error_trial(i);
               end
           end
                      
           % [u_s,v] = value_check(u_s,[u_min',u_max'],v,[-v_max',v_max']); % check range
        end
        
        [loo_min, ind_min] = min(loo_error);
        
        if loo_last > loo_min
            loo_record = [loo_record, loo_min];
            check_flag = 1;
            u_mu = u_s(1:m,ind_min)'; % the mean matrix used by Gaussian mixture distribution
            u_sigma(1,:,:) = u_s(m+1:end,ind_min); % the variance matrix used by Gaussian mixture distribution
            obj = gmdistribution(u_mu,u_sigma); % used to compute Mahalanobis distance
            d_mahal = mahal(obj,u); % compute the Mahalanobis distance
            phi = exp(-0.5 * sqrt(d_mahal)); % produce the regression matrix
 
            u_sel(:,n) = u_s(:,ind_min);
            P(:,n) = phi;

            phi_im = phi;
            for i = 1:n-1
                A(i,n) = P_im(:,i)'*phi;
                phi_im = phi_im - P_im(:,i) * A(i,n) ./ A(i,i);
            end
            A(n,n) = phi_im'*phi;
         
            P_im(:,n) = phi_im;
            b(n) = phi_im'*y;
            r_im = r_im0 - phi_im.^2./A(n,n);
            e_im = e_im0 - (b(n)/A(n,n))*phi_im;
              
        end
        
        if count_cycle > 5
            check_flag = 0;
        end

    end
end

    A = A(:,1:n);
    b = b(1:n);
    a = zeros(n,1);
    for k=n:-1:1
		a(k)=b(k)-A(k,k+1:n)*a(k+1:n);				% [n-k,n-k]
		a(k)=a(k)/A(k,k);							% [0,1]
	end
%    y_predict = P*a; 
%    mse(y_predict-y)
return


%% The function is to ensure the position and velocity are in the specified
%% range
function [u_o,v_o] = value_check(u_s,u_space,v,v_space)
%u_s n*s
%u_space 2*n [min max]
[n,s] = size(u_s);
for i = 1:s
    for j = 1:n
        if u_s(j,i)>u_space(j,2)
            u_s(j,i) = u_space(j,2);
        end
        if u_s(j,i) < u_space(j,1)
            u_s(j,i) = u_space(j,1);
        end
        if v(j,i)>v_space(j,2)
            v(j,i) = v_space(j,2);
        end
        if v(j,i) < v_space(j,1)
            v(j,i) = v_space(j,1);
        end
        if 0 < v(j,i) < 0.3
            v(j,i) = rand(1) * 0.5 * v_space(j,2);
        end
        if  -0.3 < v(j,i) < 0
            v(j,i) = rand(1) * 0.5 * v_space(j,1);
        end
    end
end
u_o = u_s;
v_o = v;
return
