function [index,theta,Jk_all] = ident_two_stage_loo(p,y)
%two-stage algorithm with leave one out method

%% First stage - select terms based on loo
[N,m] = size(p);  % n is the number of data samples, m is the terms of possible combination

k = 0;  % the initial order of selected terms
term_sel = [];  %the initial terms that were selected
term_pool = 1:m;  % The initial candidate term pool
R_k0 = eye(N); %initial value of residue matrix
R_k1 = eye(N);  %initial value of residue matrix 
J_k0 = mean(y.^2)+1;  %make sure the initial value of J_k0 is large than J_k1, so the following cycle can start
J_k1 = mean(y.^2);
stop_flag = 1;

while stop_flag
    J_k0 = J_k1;
    R_k0 = R_k1;
    J1_i = zeros(m-k,1);
    for i = 1:m-k
        R_temp = R_k0*p(:,term_pool(i));
        R_k1 = R_k0 - R_temp*R_temp'/(R_temp'*R_temp);
        J1_i(i) = mean((R_k1*y).^2./(diag(R_k1).^2));
    end
    [val, ind] = min(J1_i);
    J_k1 = val;
    k = k+1;
    term_sel = [term_sel,term_pool(ind)];
    term_pool(ind) = [];
    R_temp = R_k0*p(:,term_sel(k));
    R_k1 = R_k0 - R_temp*R_temp'/(R_temp'*R_temp);
    
    %%% determin the stop point  %%%%%
    if J_k0 > J_k1 && (J_k0 - J_k1)/mean(y.^2) < 1e-3
        stop_flag = 0;
    end
    if J_k0 < J_k1 && (J_k1 - J_k0)/mean(y.^2) > 1e-3
        stop_flag = 0;
    end
        
%     if J_k0 < J_k1 
%       stop_flag = 0;
%     end
end

Jk = J_k1;
Jk_test = [];
Jk_test = [Jk_test;Jk];
index = term_sel;

err_test1 = R_k1 * y;
sse_test1 = sse(err_test1);

a1 = p(:,index);
%a1_t = [a1,ones(N,1)];
wx = a1\y;
w2 = wx(1:k);
% b2 = wx(end);
a2 = a1*w2;
sse_e = sse(y-a2);
mse0=mse(y-a2);
rmse = [];
RMSE = sqrt(sse_e/N);
rmse = [rmse; RMSE];


% theta = inv(p(:,term_sel)'*p(:,term_sel))*p(:,term_sel)'*y;
%% The residue matrices of the first stage
R_sel = zeros(N,N,k+1); %the initial residue matrix R_0 was include in the matrices series
R_sel(:,:,1) = eye(N);
k = 1;
for i = term_sel(1:end)
    k = k+1;
    R_temp = R_sel(:,:,k-1)*p(:,i);
    R_sel(:,:,k) = R_sel(:,:,k-1) - R_temp*R_temp'/(R_temp'*R_temp);
end

%% save the intermediary variables
R_sel_first = R_sel;
term_pool_first = term_pool;
Jk_first = Jk;

R_sel = R_sel_first;
term_pool = term_pool_first;
Jk = Jk_first;
index = term_sel;

%% the second stage
loop_count = 1;
cycle_flag = 1;
k = length(index);
while cycle_flag == 1
    cycle_flag = 0;
    loop_count = loop_count+1;
    
    for k_recon = k-1:-1:1     %k = n+1
    
        for k_mov = k_recon:k-1
            %change the index of the interest term with its adjacent term
            index_temp = index(k_mov);
            index(k_mov) = index(k_mov+1);
            index(k_mov+1) = index_temp;
            %calculate the new residue matrix of the interest term
            R_temp = R_sel(:,:,k_mov) * p(:,index(k_mov));
            R_sel(:,:,k_mov+1) = R_sel(:,:,k_mov) - R_temp*R_temp'/(R_temp'*R_temp);
        end
        % the smallest new PRESS error of the terms in the pool   
        for i = 1:m-k
            R_temp = R_sel(:,:,k) * p(:,term_pool(i));
            R_k1 = R_sel(:,:,k) - R_temp*R_temp'/(R_temp'*R_temp);
            J1_i(i) = mean((R_k1*y).^2./(diag(R_k1).^2));
        end
        [val, ind] = min(J1_i); 
        if val < Jk
            cycle_flag = 1;
            index_temp = term_pool(ind);
            term_pool(ind) = index(k);
            index(k) = index_temp;
            Jk = val;
            R_temp = R_sel(:,:,k) * p(:,index(k));
            R_sel(:,:,k+1) = R_sel(:,:,k) - R_temp*R_temp'/(R_temp'*R_temp);
            RMSE = sqrt(y'*R_sel(:,:,k+1)*y/N);
        end 
    end
    Jk_test(loop_count) = Jk;
    rmse = [rmse; RMSE];
end

% theta = inv(p(:,index)'*p(:,index))*p(:,index)'*y;

%% forward selection again
p = p(:,index);
[N,m] = size(p);  % n is the number of data samples, m is the terms of possible combination
k = 0;  % the initial order of selected terms
term_sel = [];  %the initial terms that were selected
term_pool = 1:m;  % The initial candidate term pool
R_k0 = eye(N); %initial value of residue matrix
R_k1 = eye(N);  %initial value of residue matrix 
J_k0 = mean(y.^2)+1;  %make sure the initial value of J_k0 is large than J_k1, so the following cycle can start
J_k1 = mean(y.^2); 
Jk_all = zeros(m,1);
stop_flag = 1;
%J_k0 > J_k1 &&
while  stop_flag
    if k == m
        break;
    end
    
    J_k0 = J_k1;
    R_k0 = R_k1;
    J1_i = zeros(m-k,1);
    for i = 1:m-k
        R_temp = R_k0*p(:,term_pool(i));
        R_k1 = R_k0 - R_temp*R_temp'/(R_temp'*R_temp);
        J1_i(i) = mean((R_k1*y).^2./(diag(R_k1).^2));
    end
    [val, ind] = min(J1_i);
    J_k1 = val;
    k = k+1;
    Jk_all(k) = J_k1;
    term_sel = [term_sel,term_pool(ind)];
    term_pool(ind) = [];
    R_temp = R_k0*p(:,term_sel(k));
    R_k1 = R_k0 - R_temp*R_temp'/(R_temp'*R_temp);
    
        %%% determin the stop point  %%%%%
    if J_k0 > J_k1 && (J_k0 - J_k1)/mean(y.^2) < 1e-5
        stop_flag = 0;
    end
    if J_k0 < J_k1 && (J_k1 - J_k0)/mean(y.^2) > 1e-2
        stop_flag = 0;
    end
%     
%     if J_k0 < J_k1 
%       stop_flag = 0;
%     end
end

plot(Jk_all)
%  Jk = J_k1;
index = index(term_sel);
theta = (p(:,term_sel)'*p(:,term_sel))\p(:,term_sel)'*y;

err_test = R_k1 * y;
sse_test = sse(err_test);
RMSE = sqrt(sse_test/N);
sse_test

% 
% R0 = eye(N);
% for i = 1:k
%     R_temp = R0 * p(:,index(i));
%     R1 = R0 - R_temp*R_temp'/(R_temp'*R_temp);
%     Jk_all(i) = mean((R1*y).^2./(diag(R1).^2));
%     R0 = R1;
% end
    



















