function [index,a,J_record,term_record, sse_record] = ident_two_stage_loo_new(P,t)
%two-stage algorithm with leave one out method
%output: index contains the terms that are selected by the algorithm
%output: a is the associated parameters
%output: J_record records the decrease of LOO error
%output: term_record records the change of terms in each cycle of refinement
%output: procedure
%Input: P is the regression matrix
% t is the output data
% note: this code is for multi-input and single output model only.

%% First stage - select terms based on loo
% load code_test.mat

[N,m] = size(P);  % n is the number of data samples, m is the terms of possible combination
y = t;
A=zeros(m,m);
a = zeros(m,1);
k = 0;  % the initial order of selected terms
iOrder=1:m;  %the initial terms that were selected
term_pool = 1:m;  % The initial candidate term pool
J_0 = mean(y.^2)+0.00001;  %make sure the initial value of J_k0 is large than J_k1, so the following cycle can start
J_1 = mean(y.^2);
J_record = []; % to record the loo error at each step
stop_flag = 1;
k = 0;
Ay=y'*P;	
Ad=sum(P.*P);
P_im = P;
d_im = ones(N,m) - P_im.^2./(ones(N,1)*Ad);
e_im = y*ones(1,m)- P_im.*(ones(N,1)*(Ay./Ad));

while stop_flag
    k = k+1;
    J_0 = J_1;
    reod=iOrder(k:m);
    J = sum((e_im(:,reod)./d_im(:,reod)).^2)/N;	% (m-k+1)*[0,2]
	[fMaxReduce,iMax]=min(J);
    reod([1,iMax])=reod([iMax,1]);    %exchange index position
    J_1 = fMaxReduce;
    J_record = [J_record, J_1];
	cs=reod(1);
	reod=reod(2:end);                           %the left model terms
	s=P(:,cs)'*P(:,reod);						% (m-k)*[N-1,N]   %%%phi_k^T*phi_i not including the selected kth term
	for q=1:k-1
		s=s-A(q,cs)/A(q,iOrder(q))*A(q,reod);	% [m-k,m-k+1] % to evaluation a_ki  A(q,cs)=ajj
	end
	A(k,[cs,reod])=[Ad(cs),s];               % resume orginal position
        
        %%%%%%%%%%%%%%up-compute kth A %%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%down-k+1%%%%%%%%%%%%%%%%%%%%%%%
		iOrder(k:m)=[cs,reod];                      % sort-seleted index remaining index
        
		if k<m
			s=Ay(cs)/A(k,cs)*A(k,reod);				% [0,m-k+1]
			Ay(reod)=Ay(reod)-s;					% [m-k,0]
			s=A(k,reod).*A(k,reod)/A(k,cs);			% [0,m-k+1]
			Ad(reod)=Ad(reod)-s;				% [m-k,0]
            P_im(:,reod) = P_im(:,reod) - P_im(:,cs)*(A(k,reod)/A(k,cs));
            d_im(:,reod) = d_im(:,cs)*ones(1,m-k) - P_im(:,reod).^2./(ones(N,1)*Ad(reod));
            e_im(:,reod) = e_im(:,cs)*ones(1,m-k)- P_im(:,reod).*(ones(N,1)*(Ay(reod)./Ad(reod)));
        end
        
         %%% determin the stop point  %%%%%
    if J_0 > J_1 && (J_0 - J_1)/mean(y.^2) < 1e-3 && k>1
        stop_flag = 0;
    end
    if J_0 < J_1 && (J_1 - J_0)/mean(y.^2) > 1e-3 && k>1
        stop_flag = 0;
    end
%         if k > 7
%             stop_flag = 0;
%         end
end       


     n = k;   
     dn = d_im(:,iOrder(n));
     en = e_im(:,iOrder(n));
%     A = A(1:n,:);
%     a = a(1:n);
% 	for k=n:-1:1
% 		cs=iOrder(k);  % the final order
% 		reod=iOrder(k+1:n);
% 		a(k)=Ay(cs)-A(k,reod)*a(k+1:n);				% [n-k,n-k]
% 		a(k)=a(k)/A(k,cs);							% [0,1]
%     end

%% the second stage
loop_count = 0; % record how many review loops have been executed in this stage
cycle_flag = 1;
J_record = J_0;
term_record = [];
sse_record  = [];
sse_record = [sse_record,sqrt(sum(en.^2)/N)];
term_record = [term_record;iOrder(1:n)]; % record the changes of selected terms
while cycle_flag
    cycle_flag = 0;
    loop_count = loop_count+1;
    
     for k = n-1:-1:1         % move k-th term to the last order
            reod = iOrder(n+1:end);
            for q = k:n-1  %continues shift two adjacent terms
                A_q = zeros(1,m); %to store the new q-th row of matrix A
                A_q1 = A_q; %to store the new q+1-th row of matrix A
                
                A_q(iOrder(q+1)) = A(q+1,iOrder(q+1)) + A(q,iOrder(q+1))^2/A(q,iOrder(q));     % new a_{q,q}           
                A_q(iOrder(q)) = A(q,iOrder(q+1));     %update new a_{q,q+1}
                A_q(iOrder(q+2:m)) = A(q+1,iOrder(q+2:m)) + A(q, iOrder(q+1))/A(q,iOrder(q))*A(q,iOrder(q+2:m));     % new a_{q,j} j from q+1 to m             
                A_q1(iOrder(q)) = A(q,iOrder(q)) - A(q,iOrder(q+1))^2/A_q(iOrder(q+1));     % new a_{q+1,q+1}
                A_q1(iOrder(q+2:m)) = A(q, iOrder(q+2:m)) - A(q, iOrder(q+1))/A_q(iOrder(q+1)) * A_q(iOrder(q+2:m));     % new a_{q+1,j} j from q+1 to m
                
                Ay_q = Ay(iOrder(q+1)) + A(q, iOrder(q+1)) * Ay(iOrder(q))/A(q,iOrder(q));     %new ay_q
                Ay_q1 = Ay(iOrder(q)) - A(q, iOrder(q+1)) * Ay_q /A_q(iOrder(q+1));     %new ay_{q+1}
                
                P_q = P_im(:,iOrder(q+1)) + A(q, iOrder(q+1))/A(q, iOrder(q))*P_im(:,iOrder(q)); %new P_q^{(q-1)}
                P_q1 = P_im(:,iOrder(q)) - A(q, iOrder(q+1))/A_q(iOrder(q+1))*P_q; %new P_{q+1}^{(q)}
                              
                A([q,q+1],:) = [A_q;A_q1];
                Ay(iOrder([q+1,q])) = [Ay_q,Ay_q1];
                P_im(:,iOrder([q+1,q])) = [P_q,P_q1];
                
                iOrder([q,q+1]) = iOrder([q+1,q]); %shift two adjacent terms  
                Ad(iOrder(1:n)) = diag(A(1:n,iOrder(1:n)));
            end
            Ad_n = Ad;
            Ay_n = Ay;
            P_n = P_im;

            Ad_n(reod) = Ad(reod) + A(n,reod).^2/Ad(iOrder(n));
            Ay_n(reod) = Ay(reod) + Ay(iOrder(n))/Ad(iOrder(n))*A(n,reod);
            P_n(:,reod) = P_im(:,reod) + P_im(:,iOrder(n))*(A(n,reod)/A(n,iOrder(n)));
            d_n(:,reod) = dn*ones(1,m-n) + P_im(:,iOrder(n)).^2/A(n,iOrder(n))*ones(1,m-n) - P_n(:,reod).^2./(ones(N,1)*Ad_n(reod));
            e_n(:,reod) = en*ones(1,m-n) + Ay(iOrder(n))/A(n,iOrder(n))*P_im(:,iOrder(n))*ones(1,m-n)- P_n(:,reod).*(ones(N,1)*(Ay_n(reod)./Ad_n(reod)));
            
            J_pool = sum((e_n(:,reod)./d_n(:,reod)).^2)/N;	% (m-k+1)*[0,2]
            [fMaxReduce,iMax]=min(J_pool);
            
        if J_0 > fMaxReduce
                J_0 = fMaxReduce;
                cycle_flag = 1;
                J_record = [J_record,fMaxReduce];
                
                cs = reod(iMax); % candidate selected
                A_n = zeros(1,m); %to store the new q-th row of matrix A
                reod(iMax) = [];
                
                en = e_n(:,cs);
                dn = d_n(:,cs);
                sse_record = [sse_record,sqrt(sum(en.^2)/N)];
                A_n(cs) = Ad_n(cs);     %new a_{n,n}
                A_n(iOrder(n)) = A(n, cs);     %new a_{n,cs}
                A_n(reod) = P(:,cs)'*P(:,reod);     %new a_{n,j} for j = n+1:m, j!=n+iMax;
                for q = 1:n-1
                    A_n(reod) = A_n(reod) - A(q,cs)/A(q,iOrder(q))*A(q,reod);
                end               
                Ad(iOrder(n)) = Ad(iOrder(n)) - A_n(iOrder(n))^2/A_n(cs);     %new a_{cs,cs}
                Ad(reod) = Ad_n(reod) - A_n(reod).^2/A_n(cs);     %new a_{j,j} for j = n+1:m, j!=n+iMax;
      
                Ay(cs) = Ay_n(cs);
                Ay(iOrder(n)) = Ay(iOrder(n)) - A_n(iOrder(n)) * Ay(cs) / A_n(cs);     %new ay_cs
                Ay(reod) = Ay_n(reod) - Ay(cs)/A_n(cs) * A_n(reod);     %new ay_j for j = n+1:m, j!=n+iMax;
                
                P_im(:,iOrder(n)) =  P_im(:,iOrder(n)) - A_n(iOrder(n))/A_n(cs)*P_n(:,cs);
                P_im(:,reod) =  P_n(:,reod) - P_n(:,cs)*A_n(reod)/A_n(cs);
                P_im(:,cs) = P_n(:,cs);

                A(n,:) = A_n;
       
                iOrder([n,n+iMax]) = iOrder([n+iMax,n]); %shift two adjacent terms  
                Ad(iOrder(1:n)) = diag(A(1:n,iOrder(1:n)));
                term_record = [term_record;iOrder(1:n)];
            end
     end
end

index = iOrder(1:n);
%% forward selection again
P = P(:,index);
[N,m] = size(P);  % n is the number of data samples, m is the terms of possible combination
k = 0;  % the initial order of selected terms
iOrder = 1:m;  % The initial candidate term pool
J_0 = mean(y.^2)+0.00011;  %make sure the initial value of J_k0 is large than J_k1, so the following cycle can start
J_1 = mean(y.^2); 
stop_flag = 1;
J_third = [];
A=zeros(m,m);
Ay=y'*P;	
Ad=sum(P.*P);
P_im = P;
d_im = ones(N,m) - P_im.^2./(ones(N,1)*Ad);
e_im = y*ones(1,m)- P_im.*(ones(N,1)*(Ay./Ad));
while  stop_flag
    if k == m
        break;
    end
    
    k = k+1;
    J_0 = J_1;
    reod=iOrder(k:m);
    J = sum((e_im(:,reod)./d_im(:,reod)).^2)/N;	% (m-k+1)*[0,2]
	[fMaxReduce,iMax]=min(J);
    reod([1,iMax])=reod([iMax,1]);    %exchange index position
    J_1 = fMaxReduce;
    J_third = [J_third,J_1];
	cs=reod(1);
	reod=reod(2:end);                           %the left model terms
	s=P(:,cs)'*P(:,reod);						% (m-k)*[N-1,N]   %%%phi_k^T*phi_i not including the selected kth term
	for q=1:k-1
		s=s-A(q,cs)/A(q,iOrder(q))*A(q,reod);	% [m-k,m-k+1] % to evaluation a_ki  A(q,cs)=ajj
	end
	A(k,[cs,reod])=[Ad(cs),s];               % resume orginal position
        
        %%%%%%%%%%%%%%up-compute kth A %%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%down-k+1%%%%%%%%%%%%%%%%%%%%%%%
		iOrder(k:m)=[cs,reod];                      % sort-seleted index remaining index
        
		if k<m
			s=Ay(cs)/A(k,cs)*A(k,reod);				% [0,m-k+1]
			Ay(reod)=Ay(reod)-s;					% [m-k,0]
			s=A(k,reod).*A(k,reod)/A(k,cs);			% [0,m-k+1]
			Ad(reod)=Ad(reod)-s;				% [m-k,0]
            P_im(:,reod) = P_im(:,reod) - P_im(:,cs)*(A(k,reod)/A(k,cs));
            d_im(:,reod) = d_im(:,cs)*ones(1,m-k) - P_im(:,reod).^2./(ones(N,1)*Ad(reod));
            e_im(:,reod) = e_im(:,cs)*ones(1,m-k)- P_im(:,reod).*(ones(N,1)*(Ay(reod)./Ad(reod)));
        end
        
         %%% determin the stop point  %%%%%
    if J_0 > J_1 && (J_0 - J_1)/mean(y.^2) < 1e-3 && k>1
        stop_flag = 0;
    end
    if J_0 < J_1 && (J_1 - J_0)/mean(y.^2) > 1e-3 && k>1
        stop_flag = 0;
    end
end

n = k;
plot(J_third);
index = index(iOrder(1:n));

    A = A(1:n,iOrder(1:n));
    a = zeros(n,1);
    Ay = Ay(iOrder(1:n));
	for k=n:-1:1
        reod = k+1:n;
        a(k)=Ay(k)-A(k,reod)*a(k+1:n);				% [n-k,n-k]
        a(k)=a(k)/A(k,k);							% [0,1]
    end
    A = [A,Ay'];

    



















