function [index,theta,Jk_all] = ident_two_stage_reg2(p,y)
% FRA algorithm with leave one out method


[N,m] = size(p);  % n is the number of data samples, m is the terms of possible combination

val_Lambda = 0.001;  %the initial value of regularisation parameters
Lambda = val_Lambda * eye(m);  %the initial value of regularisation parameters
pre_term = 10; %the maximum cycles to updating regularisation parameters

iteration_flag = 1; % the number of cycles to update regularisation parameters
iteration_count = 0; %to count the cycles of updating regularisation parameters


while iteration_flag

    m = size(p,2);
    stop_flag = 1;
    k = 0;  % the initial order of selected terms
    term_sel = [];  %the initial terms that were selected
    term_pool = 1:m;  % The initial candidate term pool
    R_k0 = eye(N); %initial value of residue matrix
    R_k1 = eye(N);  %initial value of residue matrix 

    sse_y = y'*y;  % initial value of SSE
    p_aic = 2;  % the paremater in AIC calculation
    diag_inv_M = []; %initial value of diagonal elements of the inverse information matrix M_k^{-1}
    theta = [];  %initial the model parameters

    aic_0 = N*log((y'*y)/N); %used to store the previous AIC value
    aic_y = N*log((y'*y)/N); % the current AIC value
    aic_init = aic_0;  % the initial AIC value, used to calculate the significance of reduction as the Denominator
    Rsse_y = []; % used to store the cost function value for comparision 
    
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% %%%%%%%% the first stage %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
while stop_flag
    
    R_k0 = R_k1; %update the residual matrix
    aic_0 = aic_y; %update the AIC value
    J1_i = []; %initiate the net contribution value
    
    %calculate the contribution of each term in the term pool
    for i = 1:m-k
        R_temp = R_k0*p(:,term_pool(i));
        R_k1 = R_k0 - R_temp*R_temp'/(p(:,term_pool(i))'*R_temp + Lambda(term_pool(i),term_pool(i)));
        J1_i(i) = (y'*R_temp)^2/(p(:,term_pool(i))'*R_temp + Lambda(term_pool(i),term_pool(i)));
    end
    
    [val, ind] = max(J1_i);  %find the term that gives maximum contribution and its index
    
    k = k+1; %increase the value of terms selected
    term_sel = [term_sel,term_pool(ind)]; % store the terms that have been selected
    term_pool(ind) = []; %delete the term that selected from term pool
    
    %calculate the model parameters and SSE
    R_temp = R_k0*p(:,term_sel(k)); 
    R_k1 = R_k0 - R_temp*R_temp'/(p(:,term_sel(k))'*R_temp + Lambda(term_sel(k),term_sel(k)));
    theta = Lambda(term_sel(1:k),term_sel(1:k))\p(:,term_sel(1:k))'*R_k1*y;
    y_predict = p(:,term_sel)*theta;
    sse_y = sse(y - y_predict);
    rsse_y = y'*R_k1*y;
    
    %%updating the diagonal elements of matrix M_k
    if val_Lambda ~= 0   %if the initial value of regularisation parameter is zero, no need to update inverse of information matrix    
        if k == 1;
            diag_inv_M = 1/(p(:,term_sel(k))'*p(:,term_sel(k)));
        else        
            diag_inv_M = diag_inv_M + (Lambda(term_sel(1:k-1),term_sel(1:k-1))...
                \p(:,term_sel(1:k-1))' * R_temp).^2 ./(p(:,term_sel(k))' * ...
                R_temp + Lambda(term_sel(k),term_sel(k)));
            diag_inv_M = [diag_inv_M; 1/(p(:,term_sel(k))' * R_temp + Lambda(term_sel(k),term_sel(k)))];
        end
    end
    
    %%calculate the new AIC value
     aic_y = N*log(sse_y/N)+p_aic*k;
     aicy(k) = aic_y; %store AIC value for ploting
     
     if k > m-1  % if all terms are checked, then stop algorithm
         break;
     end
     
     %%% determin the stop point  %%%%%
    if aic_0 > aic_y && abs((aic_0 - aic_y)/aic_init)  < 1e-5
        stop_flag = 0;
    end
    if aic_0 < aic_y && abs((aic_y - aic_0)/aic_init)  > 1e-5
        stop_flag = 0;
    end
%     if k == pre_term;
%      stop_flag = 0;
%     end
     
end

%% The residue matrices of the first stage

R_sel = zeros(N,N,k+1); %the initial residue matrix R_0 was include in the matrices series
R_sel(:,:,1) = eye(N);
k = 1;
for i = term_sel(1:end)
    k = k+1;
    R_temp = R_sel(:,:,k-1)*p(:,i);
    R_sel(:,:,k) = R_sel(:,:,k-1) - R_temp*R_temp'/(p(:,i)'*R_temp + Lambda(i,i));
end

%% save the intermediary variables
R_sel_first = R_sel;
term_pool_first = term_pool;
index = term_sel;
Rsse_y = [Rsse_y, rsse_y];

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% the second stage %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
loop_count = 1;
cycle_flag = 1;
k = length(index);
while cycle_flag
    cycle_flag = 0;
    loop_count = loop_count+1;
    
    for k_recon = k-1:-1:1     %k = n+1
    
        for k_mov = k_recon:k-1
            %change the index of the interest term with its adjacent term
            index_temp = index(k_mov);
            index(k_mov) = index(k_mov+1);
            index(k_mov+1) = index_temp;
            %calculate the new residue matrix of the interest term
            R_temp = R_sel(:,:,k_mov) * p(:,index(k_mov));
            R_sel(:,:,k_mov+1) = R_sel(:,:,k_mov) - R_temp*R_temp'/(p(:,index(k_mov))'*R_temp + Lambda(index(k_mov),index(k_mov)));
        end
        % the new contribution of shifted term 
        R_temp = R_sel(:,:,k) * p(:,index(k));
        J_old = (y'*R_temp)^2/(p(:,index(k))'*R_temp + Lambda(index(k),index(k)));
        
        %the contribution of regressors in term pool
        for i = 1:m-k
            R_temp = R_sel(:,:,k) * p(:,term_pool(i));
            R_k1 = R_sel(:,:,k) - R_temp*R_temp'/(p(:,term_pool(i))'*R_temp + Lambda(term_pool(i),term_pool(i)));
            J_new(i) = (y'*R_temp)^2/(p(:,term_pool(i))'*R_temp + Lambda(term_pool(i),term_pool(i)));
        end
        [val, ind] = max(J_new); 
        if val > J_old
            cycle_flag = 1;
            index_temp = term_pool(ind);
            term_pool(ind) = index(k);
            index(k) = index_temp;
            R_temp = R_sel(:,:,k) * p(:,index(k));
            R_sel(:,:,k+1) = R_sel(:,:,k) - R_temp*R_temp'/(p(:,index(k))'*R_temp + Lambda(index(k),index(k)));
            rsse_y = y'*R_sel(:,:,k+1)*y;
        end 
    end
    Rsse_y  = [Rsse_y ; rsse_y];
end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% forward selection again %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m = k;
k = 0;  % the initial order of selected terms
term_sel = [];  %the initial terms that were selected
term_pool = index;  % The initial candidate term pool
R_k0 = eye(N); %initial value of residue matrix
R_k1 = eye(N);  %initial value of residue matrix 
stop_flag = 1;
diag_inv_M = []; %initial value of diagonal elements of the inverse information matrix M_k^{-1}
theta = [];  %initial the model parameters
aic_0 = N*log((y'*y)/N); %used to store the previous AIC value
aic_y = N*log((y'*y)/N); % the current AIC value
aic_init = aic_0;  % the initial AIC value, used to calculate the significance of reduction as the Denominator

while stop_flag
    
    if k == m
        break;
    end
    
    R_k0 = R_k1; %update the residual matrix
    aic_0 = aic_y; %update the AIC value
    J1_i = []; %initiate the net contribution value
    
    %calculate the contribution of each term in the term pool
    for i = 1:m-k
        R_temp = R_k0*p(:,term_pool(i));
        R_k1 = R_k0 - R_temp*R_temp'/(p(:,term_pool(i))'*R_temp + Lambda(term_pool(i),term_pool(i)));
        J1_i(i) = (y'*R_temp)^2/(p(:,term_pool(i))'*R_temp + Lambda(term_pool(i),term_pool(i)));
    end
    
    [val, ind] = max(J1_i);  %find the term that gives maximum contribution and its index
    
    k = k+1; %increase the value of terms selected
    term_sel = [term_sel,term_pool(ind)]; % store the terms that have been selected
    term_pool(ind) = []; %delete the term that selected from term pool
    
    %calculate the model parameters and SSE
    R_temp = R_k0*p(:,term_sel(k)); 
    R_k1 = R_k0 - R_temp*R_temp'/(p(:,term_sel(k))'*R_temp + Lambda(term_sel(k),term_sel(k)));
    theta = Lambda(term_sel(1:k),term_sel(1:k))\p(:,term_sel(1:k))'*R_k1*y;
    y_predict = p(:,term_sel)*theta;
    sse_y = sse(y - y_predict);
    rsse_y = y'*R_k1*y;
    
    %%updating the diagonal elements of matrix M_k
   if val_Lambda ~= 0   %if the initial value of regularisation parameter is zero, no need to update inverse of information matrix    
        if k == 1;
            diag_inv_M = 1/(p(:,term_sel(k))'*p(:,term_sel(k)));
        else        
            diag_inv_M = diag_inv_M + (Lambda(term_sel(1:k-1),term_sel(1:k-1))...
                \p(:,term_sel(1:k-1))' * R_temp).^2 ./(p(:,term_sel(k))' * ...
                R_temp + Lambda(term_sel(k),term_sel(k)));
            diag_inv_M = [diag_inv_M; 1/(p(:,term_sel(k))' * R_temp + Lambda(term_sel(k),term_sel(k)))];
        end
    end
    
    %%calculate the new AIC value
     aic_y = N*log(sse_y/N)+p_aic*k;
     aicy(k) = aic_y; %store AIC value for ploting
     
     if k > m-1  % if all terms are checked, then stop algorithm
         break;
     end
     
     %%% determin the stop point  %%%%%
    if aic_0 > aic_y && abs((aic_0 - aic_y)/aic_init)  < 1e-5
        stop_flag = 0;
    end
    if aic_0 < aic_y && abs((aic_y - aic_0)/aic_init)  > 1e-5
        stop_flag = 0;
    end
%     if k == pre_term;
%      stop_flag = 0;
%     end
     
end

% theta = (Lambda(term_sel(1:k),term_sel(1:k))\p(:,term_sel(1:k))*R_temp*y;
Lambda_0 = Lambda;
iteration_count = iteration_count+1;
%%% updating the regularisation parameters

 %calculate the associate value of each 
 %selected term, if this value is near 1, this term is important, if it is
 %near 0, then this term is insignificant.
gamma = ones(k,1) - diag(Lambda(term_sel,term_sel)).*diag_inv_M;

Gamma = sum(gamma); %the effective number of terms

% update the regularisation parameters for each selected terms
for i = 1:k
    Lambda(term_sel(i),term_sel(i)) = gamma(i)*sse_y/(N - Gamma)/theta(i)^2;
end

% if norm(Lambda_0 - Lambda) < 0.001
%     iteration_flag = 0;
% elseif iteration_count == 10
%     iteration_flag = 0;
% else 
%     iteration_flag = 1;
% end


end

        
        
        
        
        
        
        